module Puzzles.Day1 exposing (puzzle1, puzzle2)


puzzle1 : String -> Result String String
puzzle1 input =
    input
        |> String.split "\n"
        |> List.map puzzle1Nums
        |> List.indexedMap
            (\i items ->
                case items of
                    head :: tail ->
                        Ok (calibrationValue head tail)

                    [] ->
                        Err i
            )
        |> List.foldl
            (\value sum ->
                case ( value, sum ) of
                    ( _, Err _ ) ->
                        sum

                    ( Err i, Ok _ ) ->
                        Err ("Line" ++ (String.fromInt <| i + 1) ++ "does not contain any numbers")

                    ( Ok a, Ok b ) ->
                        Ok (a + b)
            )
            (Ok 0)
        |> Result.map String.fromInt


puzzle1Nums : String -> List Int
puzzle1Nums s =
    s
        |> String.toList
        |> List.map String.fromChar
        |> List.filterMap String.toInt


calibrationValue : Int -> List Int -> Int
calibrationValue head tail =
    tail
        |> List.reverse
        |> List.head
        |> Maybe.withDefault head
        |> (+) (head * 10)


puzzle2 : String -> Result String String
puzzle2 input =
    input
        |> String.replace "zero" "z0ero"
        |> String.replace "one" "o1ne"
        |> String.replace "two" "t2wo"
        |> String.replace "three" "t3hree"
        |> String.replace "four" "f4our"
        |> String.replace "five" "f5ive"
        |> String.replace "six" "s6ix"
        |> String.replace "seven" "s7even"
        |> String.replace "eight" "e8ight"
        |> String.replace "nine" "n9ine"
        |> puzzle1
